@@ -83,7 +83,11 @@ typedef struct _MtData
 static void
 mt_main_generate_motion_event (GdkScreen *screen, gint x, gint y)
 {
-    gdk_display_warp_pointer (gdk_display_get_default (), screen, x, y);
+    GdkDevice *cp;
+
+    cp = mt_common_get_client_pointer ();
+    if (cp)
+        gdk_device_warp (cp, screen, x, y);
 }
 
 static void
@@ -138,7 +142,7 @@ mt_main_set_cursor (MtData *mt, GdkCursorType type)
         screen = gdk_display_get_screen (gdk_dpy, i);
         gdk_window_set_cursor (gdk_screen_get_root_window (screen), cursor);
     }
-    gdk_cursor_unref (cursor);
+    g_object_unref (cursor);
 }
 
 static void
@@ -161,10 +165,10 @@ mt_main_do_dwell_click (MtData *mt)
     switch (click_type)
     {
         case MT_DWELL_CLICK_TYPE_SINGLE:
-            mt_main_generate_button_event (mt, 1, CLICK, 60);
+            mt_main_generate_button_event (mt, 1, CLICK, 80);
             break;
         case MT_DWELL_CLICK_TYPE_DOUBLE:
-            mt_main_generate_button_event (mt, 1, DOUBLE_CLICK, 10);
+            mt_main_generate_button_event (mt, 1, DOUBLE_CLICK, 40);
             mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
             break;
         case MT_DWELL_CLICK_TYPE_DRAG:
@@ -179,14 +183,16 @@ mt_main_do_dwell_click (MtData *mt)
                 mt_main_generate_button_event (mt, 1, RELEASE, CurrentTime);
                 mt_main_set_cursor (mt, GDK_LEFT_PTR);
                 mt->dwell_drag_started = FALSE;
-
                 mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
             }
             break;
         case MT_DWELL_CLICK_TYPE_RIGHT:
-            mt_main_generate_button_event (mt, 3, CLICK, 10);
+            mt_main_generate_button_event (mt, 3, CLICK, 80);
             mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
             break;
+        case MT_DWELL_CLICK_TYPE_MIDDLE:
+            mt_main_generate_button_event (mt, 2, CLICK, 80);
+            mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
         default:
             g_warning ("Unknown click-type.");
             break;
@@ -234,12 +240,17 @@ mt_main_analyze_gesture (MtData *mt)
 {
     MtSettings *ms;
     GDesktopMouseDwellDirection direction;
+    GdkDevice *cp;
     gint x, y;
 
     if (mt_service_get_click_type (mt->service) == MT_DWELL_CLICK_TYPE_DRAG)
         return TRUE;
 
-    gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
+    cp = mt_common_get_client_pointer ();
+    if (!cp)
+        return FALSE;
+
+    gdk_device_get_position (cp, NULL, &x, &y);
 
     if (below_threshold (mt, x, y))
         return FALSE;
@@ -274,18 +285,24 @@ mt_main_analyze_gesture (MtData *mt)
 static void
 dwell_start_gesture (MtData *mt)
 {
+    GdkDevice *cp;
     GdkCursor *cursor;
     GdkWindow *root;
 
     if (mt->override_cursor)
     {
-        cursor = gdk_cursor_new (GDK_CROSS);
-        root = gdk_screen_get_root_window (mt_common_get_screen ());
-        gdk_pointer_grab (root, FALSE,
-                          GDK_POINTER_MOTION_MASK,
-                          NULL, cursor,
-                          gtk_get_current_event_time ());
-        gdk_cursor_unref (cursor);
+        cp = mt_common_get_client_pointer ();
+        if (cp)
+        {
+            cursor = gdk_cursor_new (GDK_CROSS);
+            root = gdk_screen_get_root_window (mt_common_get_screen ());
+            gdk_device_grab (cp, root,
+                             GDK_OWNERSHIP_NONE, FALSE,
+                             GDK_POINTER_MOTION_MASK,
+                             cursor,
+                             gtk_get_current_event_time ());
+            g_object_unref (cursor);
+        }
     }
     else
     {
@@ -299,10 +316,18 @@ dwell_start_gesture (MtData *mt)
 static void
 dwell_stop_gesture (MtData *mt)
 {
+    GdkDevice *cp;
+
     if (mt->override_cursor)
-        gdk_pointer_ungrab (gtk_get_current_event_time ());
+    {
+        cp = mt_common_get_client_pointer ();
+        if (cp)
+            gdk_device_ungrab (cp, gtk_get_current_event_time ());
+    }
     else
+    {
         mt_main_set_cursor (mt, GDK_LEFT_PTR);
+    }
 
     mt->dwell_gesture_started = FALSE;
     mt_timer_stop (mt->dwell_timer);
@@ -604,8 +629,8 @@ mt_parse_options (int *argc, char ***argv)
             N_("Time to wait before a simulated secondary click"), "[0.5-3.0]"},
         {"dwell-mode", 'm', 0, G_OPTION_ARG_STRING, &ca.mode,
             N_("Set the active dwell mode"), "[window|gesture]"},
-        {"show-ctw", 'c', 0, G_OPTION_ARG_NONE, &ca.ctw,
-            N_("Show a click-type window"), NULL},
+        {"hide-ctw", 'c', 0, G_OPTION_ARG_NONE, &ca.ctw,
+            N_("Hide the click-type window"), NULL},
         {"threshold", 't', 0, G_OPTION_ARG_INT, &ca.threshold,
             N_("Ignore small pointer movements"), "[0-30]"},
         {"geometry", 'g', 0, G_OPTION_ARG_STRING, &ca.geometry,
@@ -663,8 +688,6 @@ mt_main (int argc, char **argv, MtCliArgs cli_args)
         return;
     }
 
-    gdk_disable_multidevice ();
-
     gtk_init (&argc, &argv);
 
     sigh = mt_sig_handler_get_default ();
@@ -716,7 +739,7 @@ mt_main (int argc, char **argv, MtCliArgs cli_args)
     if (cli_args.threshold >= 0 && cli_args.threshold <= 30)
         ms->dwell_threshold = cli_args.threshold;
     if (cli_args.ctw)
-        ms->ctw_visible = cli_args.ctw;
+        ms->ctw_visible = !cli_args.ctw;
     if (cli_args.mode)
     {
         if (g_str_equal (cli_args.mode, "gesture"))
